/******************************************************************************
 *
 * FILE: dspi.c
 *
 * Description:
 *    SPI communication (Master mode send/receive)
 *
 * History:
 *    17.11.2017   PN   Initial version
 *    13.02.2018   MF   Reuse for SPC572L64
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

/* BSP support package */
#include "bsp.h"
/* SPI communication header */
#include "dspi.h"

/* ============================================================================
 *            GLOBAL VARIABLES
 * ==========================================================================*/

uint16_t g_dspi_DataToSend = 0;
uint32_t g_dspi_ReceivedData = 0;

/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: DspiIsrHandler()
 *    DSPI0 Receive FIFO Drain ISR Handler
 * ----------------------------------------------------------------------------
 */

void DspiIsrHandler(void)
{
	/* Read received SPI data */
	g_dspi_ReceivedData =  DSPI_0.POPR.R;

	if (g_dspi_ReceivedData != g_dspi_DataToSend)
	{
		while(1); /* Received data are different than sent data => STOP execution */
	}

	/* New message for send */
	g_dspi_DataToSend = (uint16_t)g_dspi_ReceivedData +1;

	/* Clear RFDF interrupt flag */
	DSPI_0.SR.R = 0x00020000;
}

/* ============================================================================
 *            DSPI example API
 * ==========================================================================*/
// ****************************************************************************
// Name:        dspi_Init
// Parameters:
//              none
// Returns:
//              none
// Description: Initialization of DSPI0. Spi mode, master, no continuous clock, Baudrate 1MHz
//							 
// ****************************************************************************
void dspi_Init(void)
{ 
  /* Configure DSPI0 */
	DSPI_0.MCR.B.MDIS = 0x0;		    /* Enable DSPI clocks.*/
	DSPI_0.MCR.B.MSTR = 0x1;		    /* Set DSPI0 as a master */
	DSPI_0.MCR.B.CONT_SCKE = 0x0;		/* Disable continuous clock */
	DSPI_0.MCR.B.PCSIS = 0x1;		    /* Inactive state of CS = 1	 */
	DSPI_0.MCR.B.DCONF = 0x0;		    /* Set SPI mode*/
	DSPI_0.MCR.B.DIS_TXF = 0x1; 		/* Disable TX fifo */
	DSPI_0.MCR.B.DIS_RXF = 0x1; 		/* Disable RX fifo */
	DSPI_0.CTAR[0].B.FMSZ = DSPI_CTAR_FMSZ; /* Frame size */
	DSPI_0.CTAR[0].B.PBR  = DSPI_CTAR_PBR; 	/* Baud Rate Prescaler */
	DSPI_0.CTAR[0].B.BR   = DSPI_CTAR_BR;   /* Baud rate scaler */
	DSPI_0.CTAR[0].B.DBR  = DSPI_CTAR_DBR;  /* Double Baud Rate */
	DSPI_0.CTAR[0].B.PCSSCK = 0x3;        /* PSC to SCK prescaler set to 7 */
	DSPI_0.CTAR[0].B.DT = 0x8;            /* Delay after transfer */

	DSPI_0.RSER.B.RFDF_RE = 1;          /* Enable interrupt reqs */

	DSPI_0.SR.B.RFDF = 0x1; 		    /* Clear flag */
	DSPI_0.MCR.B.HALT = 0x0; 		    /* Start transmit/receive */

	/* Configure SPI pins */
	bsp_pin_EnableOutput(DSPI_PIN_PD4, DSPI_PIN_SSS_SCK);
	bsp_pin_EnableOutput(DSPI_PIN_PE9, DSPI_PIN_SSS_SOUT);
	bsp_pin_EnableOutput(DSPI_PIN_PE5, DSPI_PIN_SSS_CS0);
	bsp_pin_EnableInput(DSPI_PIN_PD5);      /* DSPI0 Serial Data Input */
	SIUL2.MSCR_MUX[DSPI_PIN_MUX_PD5].B.SSS = DSPI_PIN_SSS_SIN; 	/* DSPI0_SIN functionality input */

	/* Register interrupt handler */
	bsp_isr_RegisterHandler(UC_DSPI0_ISR_RFDF, 4, DspiIsrHandler);
}

void dspi_SendData(void)
{
	uint16_t data = g_dspi_DataToSend;
	/* Clear transmission complete flag */
	DSPI_0.SR.B.TCF = 1;
	/* Send data */
	DSPI_0.PUSHR.R =  (uint32_t)(DSPI_PUSHR_SEND_CFG | data) ;
	/* wait until transfer is not complete. */
	while (DSPI_0.SR.B.TCF == 0);
	/* Clear the transfer complete flag */
	DSPI_0.SR.R = 0x80000000;
}

// End of file
